@@ -6,13 +6,12 @@
 # 3. Titterington's Method (TM)
 # 4. Starship Method (SM)
 # 5. Method of TL-moments (TL)
+# 5a. Method of L-moments (LMom) (TL with trim=0 and low and high)
 # 6. Distributional Least Absolute (DLA)
-
+# 7. Moments (Mom)
 # Written by Benjamin Dean (24/09/2010) 
-# Changes 2013,2014 Robert King
-# Example use:
-# x <- rlogis(100)
-# fit.fkml(x, method="ML")
+# Changes 2013,2014,2019 Robert King
+
 
 ########################################################################
 #                                 Fit data                             #
@@ -28,6 +27,7 @@ fit.fkml <- function(x, method="ML", t1=0, t2=0,
 if(record.cpu.time) { time.1 <- as.numeric(proc.time()[3]) 
 } else { time.1 <- "not timing"
 }
+  
 # Sample size
 n <- length(x)
 # Sort the data
@@ -37,17 +37,55 @@ x <- sort(x)
 penalty.value <- optim.penalty
 
 # Aliases
-if (method == "ML")  {method.id <- 1; method.name="Maximum Likelihood"}
-if (method == "MSP") {method.id <- 2; method.name="Maximum Spacings Product"}
-if (method == "MPS") {method.id <- 2; method.name="Maximum Product of Spacings"}
-if (method == "TM")  {method.id <- 3; method.name="Titterington's"}
-if (method == "SM")  {method.id <- 4; method.name="Starship"}
-if (method == "TL")  {method.id <- 5; method.name="Trimmed L-Moments"}
-if (method == "DLA")  {method.id <- 6; method.name="Distributional Least Absolutes"}
+method.id <- 0 # if this is not updated, no method was defined.
+if (toupper(method) == "ML")  {method.id <- 1; method.name="Maximum Likelihood"}
+if (toupper(method) == "MSP") {method.id <- 2; method.name="Maximum Spacings Product"}
+if (toupper(method) == "MPS") {method.id <- 2; method.name="Maximum Product of Spacings"}
+if (toupper(method) == "TM")  {method.id <- 3; method.name="Titterington's"}
+if (toupper(method) == "SM")  {method.id <- 4; method.name="Starship"}
+if (toupper(method) == "TL")  {method.id <- 5; method.name="Trimmed L-Moments"}
+if (toupper(method) == "LMOM")  {method.id <- 5; method.name="L-Moments"}
+if (toupper(method) == "DLA")  {method.id <- 6; method.name="Distributional Least Absolutes"}
+if (toupper(method) == "MOM") {method.id <- 7; method.name="Moments"}
 
+if (method.id == 0) {
+  stop(paste("unknown estimation method code:",method))
+}
+# t1,t2 check
+if (length(t1)>1) {
+  warning(paste("Argument t1 should be only 1 value.  It has been truncated to its first value,",t1[1]))
+  t1 = t1[1]
+}
+if (length(t2)>1) {
+  warning(paste("Argument t2 should be only 1 value.  It has been truncated to its first value,",t2[1]))
+  t2 = t2[1]
+}
+if ((t1>0)&(t1<1)) {warning(paste("Trimming arguments should be integers to give number of trimmed observations, rather than t1=",t1))}
+if ((t2>0)&(t2<1)) {warning(paste("Trimming arguments should be integers to give number of trimmed observations, rather than t2=",t2))}
+if (method.id != 5) {
+  if (t1 != 0) {
+    warning(paste("Trimming only used in trimmed L-Moments.  Lower trim of ",t1,"has been changed to zero."))
+    t1=0
+  }
+  if (t2 != 0) {
+    warning(paste("Trimming only used in trimmed L-Moments.  Upper trim of ",t2,"has been changed to zero."))
+    t2=0
+  }
+}
+# LMom is just TL with t1=0, t2=0.  Check for this
+if (method=="LMom") {
+  if (!(t1==0 && t2==0)) {
+message(paste("L-Moments method called with trimming arguments, low=",t1,"high=",t2,"\nSo, renamed to Trimmed L-Moments"))
+    method.name="Trimmed L-Moments"
+    method="TL"
+  }
+}
+if (t1+t2>(n-1)) {
+  stop(paste("No observations left in data after trimming!
+    t1=",t1,", t2=",t2,". Total data, n=",n,sep=""))
+}
 # Perform the fitting process: 
-
-if (method.id != 5) { # ML, MSP, TM, SM or DLA (not TL)
+if ((method.id != 5)&(method.id != 7)) { # ML, MSP, TM, SM or DLA (not TL, LM, Mom)
 
   # Starting values from grid search
   grid.results <- grid.search(l3.grid,l4.grid,method.id,x,n,
@@ -81,8 +119,8 @@ if (method.id != 5) { # ML, MSP, TM, SM or DLA (not TL)
   if (return.data) {result$data = x}
   class(result) <- "starship"
   names(result$lambda) <- paste("lambda",1:length(result$lambda),sep="")
-} else { # TL
-
+} else { # TL, LM, Mom
+  if (method.id==5) { # TL, LM
   # Starting values from grid search
   grid.results <- grid.search.tl(l3.grid,l4.grid,method.id,x,n,
     penalty.value,t1,t2,inverse.eps)
@@ -99,6 +137,7 @@ if (method.id != 5) { # ML, MSP, TM, SM or DLA (not TL)
   l3.est <- optim.results$par[1]; l4.est <- optim.results$par[2]
   conv <- optim.results$convergence; its <- optim.results$counts[["function"]]
   obj <- optim.results$value
+  if (obj==optim.penalty) {stop("optimisation unable to find estimate")}
   # Add the optimisation method to the list
   optim.results$optim.method = optim.method 
   
@@ -121,8 +160,23 @@ if (record.cpu.time) {time.2 <- as.numeric(proc.time()[3]); runtime <- round(tim
     method.code=method,method.name=method.name,
     fkml.oldstyle.results=old.style.results)
   if (return.data) {result$data = x}
+  if (method=="TL") {
+    result$trim = c(t1,t2,n)
+    names(result$trim) = c("lower","upper","n")
+    }
   class(result) <- "starship"
   names(result$lambda) <- paste("lambda",1:length(result$lambda),sep="")
+  } else { # This should just be moments, but let's double check
+    if (method.id==7) {
+      #  Moments - do I extend to region A estimate, region B estimate?
+      
+      result = fit.fkml.moments(x) #  This returns an object of class starship
+      if (return.data) {result$data = x}
+      # Store results
+      if (record.cpu.time) {time.2 <- as.numeric(proc.time()[3]); runtime <- round(time.2-time.1,2)} else {runtime <- NA}
+    }
+    else {stop("Unknown estimation method id")}
+  }
 }
 
 # Return result
@@ -163,10 +217,10 @@ l3 <- par[1]
 l4 <- par[2]
 
 obj.result.tl <- .C("fit_fkml",as.integer(method.id),as.double(0),
-  as.double(1),as.double(l3),as.double(l4),as.double(x),
-  as.integer(n),as.double(penalty.value),as.double(t1),as.double(t2),
-  as.double(0),as.double(1),as.integer(0),result = as.double(0),
-  as.double(inverse.eps))
+    as.double(1),as.double(l3),as.double(l4),as.double(x),
+    as.integer(n),as.double(penalty.value),as.double(t1),as.double(t2),
+    as.double(0),as.double(1),as.integer(0),result = as.double(0),
+    as.double(inverse.eps))
 
 return(obj.result.tl[["result"]])
 }
@@ -196,11 +250,12 @@ for (i in 1:length(l3.grid)) {
 return(list(response=obj.min,lambda=c(l1.s,l2.s,l3.s,l4.s)))
 }
 
-################################################################################
-#            Grid search for starting values (method of TL-moments)            #
-################################################################################
+#######################################################################
+#            Grid search for starting values (method of TL-moments)   
+####################################################################
 
 # A different grid search is also needed for the method of TL-moments.
+# This is used for untrimmed L moments, and for Moments
 
 grid.search.tl <- function(l3.grid,l4.grid,method.id,x,n,penalty.value,t1,t2,inverse.eps) {
 
