@@ -3,6 +3,7 @@ print.starship <- function(x,digits = max(3, getOption("digits") - 3), ...)
 # I should include the call
 # Add names to the vector in starship
 cat(paste(x$method.name,"estimate, gld type:",x$param,"\n"))
+if(!is.null(x$trim)){cat(paste("trimmed",x$trim[1],"low,",x$trim[2],"high. n=",x$trim[3],"\n"))}
 print.default(format(x$lambda,digits=digits), print.gap = 2,quote=FALSE)
 }
 
@@ -10,14 +11,25 @@ summary.starship <- function(object,...)
 {
 cat(paste("Generalised Lambda Distribution",object$param,"type.",
           object$method.name," estimate.\n"))
-cat("\nAdaptive Grid estimates:\n")
-fake.lambda <- object$grid.results$lambda
-names(fake.lambda) <- paste("lambda",1:length(fake.lambda),sep="")
-fake.starship.object <- list(param=object$param,lambda=fake.lambda)
-print.starship(fake.starship.object)
-cat(paste("internal g-o-f measure at grid minimum:",
-format(object$grid.results$response),"\n"))
-cat("\nOptim (final) estimates (starting from grid estimates):\n")
+if(!is.null(object$trim)){
+  cat(paste("L Moments based on data trimmed by;\n",object$trim[1],"lowest and",object$trim[2],"highest observations out of",object$trim[3]))
+}
+if(exists("object$grid.results")){
+  cat("\nAdaptive Grid estimates:\n")
+  fake.lambda <- object$grid.results$lambda
+  if (object$method.code=="TL"){
+      names(fake.lambda) <- paste("lambda",3:4)
+    } else {
+      names(fake.lambda) <- paste("lambda",1:length(fake.lambda),sep="")
+    }
+  fake.starship.object <- list(param=object$param,lambda=fake.lambda)
+  print.starship(fake.starship.object)
+  cat(paste("internal g-o-f measure at grid minimum:",
+  format(object$grid.results$response),"\n"))
+  cat("\nOptim (final) estimates (starting from grid estimates):\n")
+} else {
+  cat("\nOptim (final) estimates:\n")
+}
 print.starship(object)
 cat(paste("internal g-o-f measure at optim minimum:",
 format(object$optim.results$value),"\n"))
@@ -29,7 +41,7 @@ cat("Message: ")
 print(object$optim.results$message)
 }
 
-plot.starship <- function(x,data=NULL,ask=NULL,one.page=NULL,breaks="Sturges",plot.title="default",...)
+plot.starship <- function(x,data=NULL,ask=NULL,one.page=FALSE,breaks="Sturges",plot.title="default",...)
 {
 if (plot.title == "default") {
   plot.title <- paste(x$method.name,"fit of",x$param,"type GLD")
@@ -42,7 +54,7 @@ if (is.null(x$data)){
 	if (is.null(data)) {data <- x$data #using data returned by starship function
 		} else { 
 		warning(paste(substitute(x),"has a data element and the data argument was also given.\nUsing ",paste(substitute(data))," instead of the data element of ",substitute(x))) } }
-if (is.null(one.page)) { if (interactive()) {one.page=TRUE} else {one.page=FALSE}}
+# if (is.null(one.page)) { if (interactive()) {one.page=TRUE} else {one.page=FALSE}} # this was here so that interactive sessions didn't have a user-visible change
 if (is.null(ask)) {if (one.page) {ask=FALSE} else {
   if (interactive()) {ask=TRUE} else {ask=FALSE}
     }
@@ -52,7 +64,7 @@ if (one.page) {par(mfrow=c(2,1))}
 qqgl(y=data,lambda.pars1=x$lambda,param=x$param,xlab=paste(x$method.name," Fitted Theoretical Quantiles"),main=plot.title) # add which option here
 hist(data,prob=TRUE,xlab="Data",breaks=breaks,main=plot.title,...)
 plotgld(lambda1=x$lambda,param=x$param,new.plot=FALSE,...)
-par(opar) # Return to previous par
+if (one.page) {par(opar)} # Return to previous par
 }
 
 print.GldGPDFit <- function(x,digits = max(3, getOption("digits") - 3), ...)
